/* sysparams unit test */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <check.h>

/*** tests below are for Demo purposes only ***/
START_TEST(test_pass)
{
  fail_unless (1==1, "Shouldn't see this");
}
END_TEST

START_TEST(test_fail)
{
  fail("Failure");
}
END_TEST

HWND ghTestWnd;

/*** sysparams test ***/
DWORD WINAPI SysParamsThreadFunc(LPVOID lpParam);
LRESULT CALLBACK SysParamsTestWndProc(HWND hWnd, UINT msg, WPARAM wParam,
                                      LPARAM lParam);

int change_counter;
int change_last_action;
LPSTR change_reg_section;

static void sysparams_setup (void)
{
}

static void sysparams_teardown (void)
{
} 

START_TEST(test_sysparams)
{
    /*
    HANDLE hThread;
    DWORD dwThreadId;
    */
    MSG msg;

    WNDCLASS wc;
    HANDLE hInstance = GetModuleHandle(NULL);;

    change_counter = 0;
    change_last_action = 0;
    change_reg_section = "";

    wc.lpszClassName = "SysParamsTestClass";
    wc.lpfnWndProc = SysParamsTestWndProc;
    wc.style = CS_OWNDC | CS_VREDRAW | CS_HREDRAW;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(0, (LPCTSTR)IDI_APPLICATION);
    wc.hCursor = LoadCursor(0, (LPCTSTR) IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName = 0;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    RegisterClass(&wc);

    ghTestWnd = CreateWindow("SysParamsTestClass", "Test System Parameters Application",
                             WS_OVERLAPPEDWINDOW, 0, 0, 100, 100, 0, 0, hInstance, NULL);
    ShowWindow(ghTestWnd, SW_SHOW);

    SysParamsThreadFunc(NULL);
    /*
      hThread = CreateThread(NULL, 0, SysParamsThreadFunc, 0, 0, &dwThreadId);
 
    if (hThread == 0)
    {
        printf("CreateThread failed.\n");
        exit(1);
    }
    CloseHandle(hThread);
    */

    while(GetMessage(&msg, 0, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
}
END_TEST

LRESULT CALLBACK SysParamsTestWndProc(HWND hWnd, UINT msg, WPARAM wParam,
                                       LPARAM lParam)
{
    switch(msg) {

    case WM_SETTINGCHANGE:
	change_counter++;
	change_last_action = wParam;
	change_reg_section = (LPSTR) lParam;
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    default:
        return(DefWindowProc(hWnd, msg, wParam, lParam));
    }
    
    return 0;
}

/*
Performs testing for system parameters messages
params:
 - system parameter id
 - supposed value of the registry key
*/
void test_change_message(UINT action, char *reg_section)  {
    fail_unless( 1 == change_counter, "Each message is processed" );
    change_counter = 0;
    fail_unless( action == change_last_action, "Correct action is processed" );
    change_last_action = 0;
    fail_unless( strcmp(reg_section, change_reg_section),
                 "Unexpected registry section" );
    change_reg_section = "";
}

/**
 * Registry entries for the system parameters.
 * Names are created by 'SET' flags names.
 * We assume that corresponding 'GET' entries use the same registry keys.
 */
DWORD WINAPI SysParamsThreadFunc(LPVOID lpParam)
{
    BOOL old_b;
    BOOL b_a;
    BOOL b_w;
    BOOL curr_val;

    /********************/
    /*START_TEST( SPI_SETSCREENSAVEACTIVE );*/
    SystemParametersInfoA( SPI_GETSCREENSAVEACTIVE, 0, &old_b, 0 );

    curr_val = TRUE;
    SystemParametersInfoA( SPI_SETSCREENSAVEACTIVE, curr_val, 0,
                           SPIF_UPDATEINIFILE | SPIF_SENDCHANGE );
    /*test_change_message( SPI_SETSCREENSAVEACTIVE, "" );*/

    SystemParametersInfoA( SPI_GETSCREENSAVEACTIVE, 0, &b_a, 0 );
    fail_unless( curr_val == b_a, "Setter failured" );
    SystemParametersInfoW( SPI_GETSCREENSAVEACTIVE, 0, &b_w, 0 );
    fail_unless( curr_val == b_w, "Setter failured");

    curr_val = FALSE;
    SystemParametersInfoW( SPI_SETSCREENSAVEACTIVE, curr_val, 0,
			   SPIF_UPDATEINIFILE | SPIF_SENDCHANGE );
    /*test_change_message( SPI_SETSCREENSAVEACTIVE, "" );*/

    SystemParametersInfoA( SPI_GETSCREENSAVEACTIVE, 0, &b_a, 0 );
    fail_unless( curr_val == b_a, "Setter failured" );
    SystemParametersInfoW( SPI_GETSCREENSAVEACTIVE, 0, &b_w, 0 );
    fail_unless( curr_val == b_w, "Setter failured" );

    SystemParametersInfoA( SPI_SETSCREENSAVEACTIVE, old_b, 0, SPIF_UPDATEINIFILE );

    SendMessage(ghTestWnd, WM_DESTROY, 0, 0);
    return 0;
}

Suite *make_sysparams_suite (void)
{
    Suite *s;
    TCase *tc;

    s = suite_create("Sysparams Suite");

    tc = tcase_create("Demo Test Case");
    tcase_add_test(tc, test_pass);
    tcase_add_test(tc, test_fail);
    suite_add_tcase(s, tc);

    tc = tcase_create("Sysparams Test Case");
    tcase_add_test(tc, test_sysparams);
    tcase_add_unchecked_fixture(tc, sysparams_setup, sysparams_teardown);
    suite_add_tcase(s, tc);

    return s;
}

int main (void)
{
    SRunner *sr;
    Suite *s;

    s = make_sysparams_suite();
    sr = srunner_create(s);
    /* fork modes: CK_FORK, CK_NOFORK */
    srunner_set_fork_status(sr, CK_NOFORK);

    /*all the printmodes: CK_SILENT | CK_MINIMAL | CK_NORMAL | CK_VERBOSE*/
    srunner_run_all(sr, CK_NORMAL);
    return 0;
}
